function Trig = computeFaceNormals(Trig)

% Copyright (C) <2012>  <Aydin Varol, Mathieu Salzmann, Pascal Fua and Raquel Urtasun>
% 
% Implementation of CLVM method described in:
% A. Varol, M. Salzmann, P. Fua and R. Urtasun, A Constrained Latent Variable Model, 
% Conference on Computer Vision and Pattern Recognition, June 2012.
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Aydin Varol, CVLab-EPFL, 2012.
% aydin.varol@gmail.com, www.aydinvarol.com

nF=Trig.NFacets;
Trig.Facet_Normal = zeros(nF,3);
Trig.Facet_NormalMagnitude = zeros(nF,1);
Trig.Facet_Centroid = zeros(nF,3);

for i=1:nF
   
   ia = Trig.Facets(i,1);
   ib = Trig.Facets(i,2);
   ic = Trig.Facets(i,3);
 
   a=Trig.Coords(ia,:);
   b=Trig.Coords(ib,:);
   c=Trig.Coords(ic,:);
   
   %centroid
   centroid=(a+b+c)/3; 
   
   %normal
   ab=a-b;
   ac=a-c;
   nrm = cross(ab,ac,2);
   magnitude=norm(nrm);
   nrm=nrm/magnitude;
   
   Trig.Facet_Normal(i,:) = nrm;
   Trig.Facet_NormalMagnitude(i)=magnitude;
   Trig.Facet_Centroid(i,:)=centroid;
   
end