#include "mex.h"
#include "blas.h"
#include "lapack.h"
#include <stdlib.h>
#include <stdio.h>
#include "string.h"
#include "math.h"
#include "ssmult.h"
#include "time.h"
#include <float.h>
#include <stdio.h>


#define IsNonZero(d) ((d)!=0.0)


void displayMat(double* A,int m,int n){

  int c,r;
  int ind;

  mexPrintf("\n");
  for(r=0;r<m;r++){
    for(c=0;c<n;c++){
      ind = c*m+r;
      mexPrintf("%1.4f\t",A[ind]);
    }
    mexPrintf("\n");
  }
  mexPrintf("\n");
	
}



void printMat2File(double *A, int m, int n, char * outputFilename){

  FILE *fp = fopen(outputFilename, "w");	
  int c,r;
  int ind;

	
  for(r=0;r<m;r++){
    for(c=0;c<n;c++){
      ind = c*m+r;
      fprintf(fp,"%1.4f\t",A[ind]);
    }
    fprintf(fp,"\n");
  }

  fclose(fp);
	  
}

/* mean=mean(A) */
double  meanMatrix(double *A,int m, int n){

  double sum = 0;
  int i;
	
  for (i=0;i<m*n;i++){
    sum += A[i];
  }

  return(sum/(m*n));
		
}

/* max=max(A) */
double  maxMatrix(double *A,int m, int n){

  double maxx = -1e30;
  int i;
	
  for (i=0;i<m*n;i++){
    if(A[i]>maxx){
      maxx = A[i];
    }
  }

  return(maxx);
		
}

/* min=min(A) */
double minMatrix(double *A,int m, int n){

  double minn = 1e30;
  int i;
	
  for (i=0;i<m*n;i++){
    if(A[i]<minn){
      minn = A[i];
    }
  }

  return(minn);
}

/* id = min(A) */
int findMinMatrix(double *A,int m, int n){

  double minn = 1e30;
  int i;
  int id = 0;
	
  for (i=0;i<m*n;i++){
    if(A[i]<minn){
      minn = A[i];
      id = i;
    }
  }

  return(id);

}

void convertSparse2Full(double *Full, mxArray *Sparse, int m, int n){

  mwIndex *irs,*jcs,i,j,k;
  double *sr;
  int numCol = mxGetN(Sparse);
  int numRow,jind,iind;
  int indexfull;

  sr  = mxGetPr(Sparse);
  irs = mxGetIr(Sparse);
  jcs = mxGetJc(Sparse);
	
  int counter = 0;
  for (i=0;i<numCol;i++){
    numRow = jcs[i+1]-jcs[i];
    iind = i;
    for(j=0;j<numRow;j++){
      jind = irs[counter];
      indexfull = iind*m+jind;
      Full[indexfull] = sr[counter];          
      counter++;
    }
  }
		
}

	

/* assumes that the sparse mxarray has enough storage capacity */
void convertFull2Sparse(mxArray *Sparse, double *Full,int m, int n){
    
  mwIndex *irs,*jcs,j,k;
  double *sr;
  int isfull;

  sr  = mxGetPr(Sparse);
  irs = mxGetIr(Sparse);
  jcs = mxGetJc(Sparse);
    
  /* Copy nonzeros */
  k = 0;
  isfull=0;
  for (j=0; (j<n); j++) {
    mwSize i;
    jcs[j] = k;
    for (i=0; (i<m ); i++) {
      if (IsNonZero(Full[i])) {                               
	sr[k] = Full[i];
	irs[k] = i;
	k++;
      }
    }
    Full += m;
  }
  jcs[n] = k;
}

/*invA = inv(A), of size N x N.*/
void invertMatrix(double *invA, double *A, int N){

  mwSignedIndex *IPIV;
  mwSignedIndex prod = N * N;
  mwSignedIndex INFO;
  mwSignedIndex N2 = N;
  
  double *a2 = (double *)malloc(N*N*sizeof(double));
  memcpy(a2,A,N*N*sizeof(double));
  
  double *Awork = (double *)malloc(N*N*sizeof(double));
  
  IPIV = (mwSignedIndex*) malloc((N+1)*sizeof(mwSignedIndex));
    
  /* Pass arguments to Fortran by reference */
  dgetrf_(&N2, &N2, a2, &N2, IPIV, &INFO);   
  dgetri_(&N2,  a2 , &N2, IPIV,Awork, &prod, &INFO);
  
  memcpy(invA,a2,N*N*sizeof(double));

  free(Awork);
  free(IPIV);
  free(a2);
	
}


/* A[:,:] = val */
void setMatrixValue(double *A,int m,int n,double val){

  int i;
  for (i=0;i<m*n;i++){
    A[i] = val;
  }
	
}


/* res = Mat * vec, where Mat is a sparse matrix and vec is a dense vector */
void multSparseMatDenseVec(double* res,mxArray *Mat,double *vec){

  mwIndex *irs,*jcs,i,j,iind,jind,ind,M,N;
  int counter,numCol,numRow;
  double *sr;

  sr  = mxGetPr(Mat);
  irs = mxGetIr(Mat);
  jcs = mxGetJc(Mat);

  M = mxGetM(Mat);
  N = mxGetN(Mat);

  numCol = N;
	
  setMatrixValue(res,N,1,0);
		
  counter = 0;
  for (i=0;i<numCol;i++){
    numRow = jcs[i+1]-jcs[i];
    iind = i;
    for(j=0;j<numRow;j++){
      jind = irs[counter];
      res[jind] += sr[counter]*vec[i];
      /*
	if(i==0)
	mexPrintf("cc %d sr %f vec %f jind %d res[%d] %f\n",counter,sr[counter],vec[i],jind,jind,res[jind]);
      */
      counter++;
    }
  }

  /*mexPrintf("max of res = %f \n",maxMatrix(res,N,1));*/
}

/* res = Mat * vec, where Mat is a sparse matrix and vec is a dense vector */
void multDenseVecSparseMat(double* res,double *vec,mxArray *Mat){

  mwIndex *irs,*jcs,i,j,iind,jind,ind,M,N;
  int counter,numCol,numRow;
  double *sr;

  sr  = mxGetPr(Mat);
  irs = mxGetIr(Mat);
  jcs = mxGetJc(Mat);

  M = mxGetM(Mat);
  N = mxGetN(Mat);

  numCol = N;
	
  setMatrixValue(res,1,M,0);
	
  counter = 0;
  for (i=0;i<numCol;i++){
    numRow = jcs[i+1]-jcs[i];
    iind = i;
    for(j=0;j<numRow;j++){
      jind = irs[counter];
      res[iind] += sr[counter]*vec[jind];
      counter++;
    }
  }

}
   

/*C = A * B, A maxna B mbxnb C maxnb*/
void multMatrix(double *C, double *A, int ma, int na,double *B, int mb, int nb){

  char *chn = "N";
  /* scalar values to use in dgemm */
  double one = 1.0, zero = 0.0;

  mwSignedIndex m = ma;
  mwSignedIndex p = na;
  mwSignedIndex n = nb;
			
  /* Pass arguments to Fortran by reference */
  dgemm(chn, chn, &m, &n, &p, &one, A, &m, B, &p, &zero, C, &m);

}

/*v1 and v2 are vectors of the same size*/
double dotProduct(double *v1,double *v2, int size){
  double sum = 0;
  int i;

  for (i=0;i<size;i++){
    if(IsNonZero(v1[i]) && IsNonZero(v2[i])){
      sum += (v1[i]*v2[i]);		
    }
  }
  return sum;
  
}

/* C = A + B*scale, all of size m x n */
void addScaledMatrix(double *C,double *A, double *B, int m,int n,double scale){

  int i;
  for (i=0;i<m*n;i++){
    C[i] = A[i]+scale*B[i];
  }

}

/* B = A*scale, all of size m x n */
void scaleMatrix(double *B,double *A, int m,int n,double scale){

  int i;
  for (i=0;i<m*n;i++){
    B[i] = A[i]*scale;
  }

}




/* r is the rowIndth row of matrix A of size m x n */
void getMatrixRow(double *r, double *A, int m, int n, int rowInd){   
  int i;	
  for(i=0;i<n;i++){
    r[i]= A[i*m+rowInd];
  }  
}

/* r is the rowIndth row of matrix A of size m x n */
void setMatrixRow(double *A, double *r, int m, int n, int rowInd){   
  int i;	
  for(i=0;i<n;i++){
    A[i*m+rowInd]=r[i];
  }  
}

/* r is the colIndth colomn of matrix A of size m x n */
void getMatrixCol(double *r, double *A, int m, int n, int colInd){   
  int i;	
  for(i=0;i<m;i++){
    r[i]= A[colInd*m+i];
  }  
}

/* r is the colIndth colomn of matrix A of size m x n */
void setMatrixCol(double *A, double *r, int m, int n, int colInd){   
  int i;	
  for(i=0;i<m;i++){
    A[colInd*m+i] = r[i];
  }  
}

/* block is the block A[rmin:rmax,cmin:cmax], inclusive */
void getMatrixBlock(double *block, double *A, int m, int n, int rmin, int rmax, int cmin, int cmax){

  int r,c;
  int indlocal,indglobal;
  int mb = rmax-rmin+1;

  for(r=rmin;r<=rmax;r++){
    for(c=cmin;c<=cmax;c++){			
      indlocal = (c-cmin)*mb+(r-rmin);
      indglobal = c*m+r;					
      block[indlocal] = A[indglobal];			
    }
  }

}

   
/*v1 and v2 are vectors of the same size*/
double computeSquaredEuclidianDist(double *v1,double *v2, int size){
  double dist = 0;
  int i;

  for (i=0;i<size;i++){
    dist += (v1[i]-v2[i])*(v1[i]-v2[i]);		
  }
  return dist;	
}

/* X1 = m x d1 and X2 = m x d2 and K = d1 x d2 matrices */
void computeLinKern( double* K, double* X1, double* X2, mwSignedIndex m, mwSignedIndex d1, mwSignedIndex d2){

  int i,j;
  int ind1,ind2,indk;
  double dist;

  mxArray *v1 = mxCreateDoubleMatrix(m, 1, mxREAL);
  mxArray *v2 = mxCreateDoubleMatrix(m, 1, mxREAL);
  
	 
  for(i=0;i<d1;i++){
    for(j=0;j<d2;j++){
      indk = i*d1+j;
      getMatrixCol(mxGetPr(v1), X1, m, d1, i);
      getMatrixCol(mxGetPr(v2), X2, m, d2, j);
      K[indk] = dotProduct(mxGetPr(v1),mxGetPr(v2),m); 
    }
  }

  mxDestroyArray(v1);
  mxDestroyArray(v2);

}
	

/* X1 = m x d1 and X2 = m x d2 and K = d1 x d2 matrices */
void computeRBFKern( double* K, double* X1, double* X2, mwSignedIndex m, mwSignedIndex d1, mwSignedIndex d2, double invWidth){

  int i,j;
  int ind1,ind2,indk;
  double dist;

  mxArray *v1 = mxCreateDoubleMatrix(m, 1, mxREAL);
  mxArray *v2 = mxCreateDoubleMatrix(m, 1, mxREAL);
  
	 
  for(i=0;i<d1;i++){
    for(j=0;j<d2;j++){
      indk = i*d1+j;
      getMatrixCol(mxGetPr(v1), X1, m, d1, i);
      getMatrixCol(mxGetPr(v2), X2, m, d2, j);
      dist =  computeSquaredEuclidianDist(mxGetPr(v1),mxGetPr(v2), m);
      K[indk] = exp(-dist*0.5*invWidth);
      /*K[indk] = dist;*/
    }
  }

  mxDestroyArray(v1);
  mxDestroyArray(v2);

}

/* A = B' */
void transposeMatrix(double *A,double *B,int m,int n){

  int c,r;
  int inds,indd; 

  for(c=0;c<n;c++){
    for(r=0;r<m;r++){			
      inds = c*m+r;
      indd = r*n+c;
      A[indd]=B[inds];
    }		
  }	
}

/* Adest = Bsource */
void cloneMatrix(double *Adest,double *Bsource,int m,int n){
    
  int i;
  for(i=0;i<m*n;i++){
    Adest[i] = Bsource[i];
  }

}


/* A = [B,C] */
void horzCatMatrices(double *A, double* B, int m1, int n1, double* C, int m2, int n2){ 
  

  int bytes_to_copy1 = m1*n1*sizeof(double);
  int bytes_to_copy2 = m2*n2*sizeof(double); 

  memcpy(A,B,bytes_to_copy1);
  memcpy(A+m1*n1,C,bytes_to_copy2);

}

/* A = [B;C] */
void vertCatMatrices(double *A, double* B, int m1, int n1, double* C, int m2, int n2){

  mxArray *Bt = mxCreateDoubleMatrix(n1, m1, mxREAL);
  transposeMatrix(mxGetPr(Bt),B,m1,n1);

  /*displayMat(mxGetPr(Bt),n1,m1);*/

  mxArray *Ct = mxCreateDoubleMatrix(n2, m2, mxREAL);
  transposeMatrix(mxGetPr(Ct),C,m2,n2);

  /*displayMat(mxGetPr(Ct),n2,m2);*/

  mxArray *At = mxCreateDoubleMatrix(n1,(m1+m2),mxREAL);
  horzCatMatrices(mxGetPr(At),mxGetPr(Bt),n1,m2,mxGetPr(Ct),n2,m2);

  /*displayMat(mxGetPr(At),n1,(m1+m2));*/

  transposeMatrix(A,mxGetPr(At),n1,(m1+m2));
  /*displayMat(A,(m1+m2),n1);*/

  mxDestroyArray(At);
  mxDestroyArray(Bt);
  mxDestroyArray(Ct);
	
}


void updateConstDerivativesAndValues(mxArray *G,double *Ct,double *Yhat, double *L02,mxArray **FQ, mwSignedIndex NC,mwSignedIndex V,mwSignedIndex D){

  int v,c,indl;
  double yfqyhat;
  double *yfq = (double*)malloc(1*D*sizeof(double));
  double *ytemp = (double*)malloc(1*D*sizeof(double));
  double *yfqScaled = (double*)malloc(1*D*sizeof(double));	
	
		
  for (v=0;v<V;v++){

    getMatrixRow(ytemp, Yhat, V, D, v);

    /*debug*/
    /*if(v==0)
      displayMat(ytemp,1,D);*/		

				
    for (c=0;c<NC;c++){
            
      indl = v*NC+c;
      multMatrix(yfq,ytemp,1,D,mxGetPr(FQ[c]),D,D);
			
      scaleMatrix(yfqScaled,yfq,1,D,2);

      /*if(v==0 && c==0)
	displayMat(yfqScaled,1,D);*/
            
      setMatrixRow(mxGetPr(G),yfqScaled,NC*V,D,indl);

      /*if(v==0 && c==0)
	mexPrintf("ct(1) = %f \n",dotProduct(ytemp,yfq,D));*/

      Ct[indl] = dotProduct(ytemp,yfq,D) - L02[c];

      /*if(v==0 && c==0)
	mexPrintf("ct(indl) = %f \n",Ct[indl]);*/
            
    }
  }

  /*
    printMat2File(mxGetPr(G),NC*V,D,"G.txt");
    printMat2File(Ct,NC*V,1,"Ct.txt");
  */

	
  free(yfq);
  free(ytemp);
  free(yfqScaled);

}


void updateConstDerivativesAndValues2(mxArray *G,double *Ct,double *Yhat, double *L02,mxArray **FQ, mwSignedIndex NC,mwSignedIndex V,mwSignedIndex D){

  int v,c,indl;
  double yfqyhat;
  double *yfq = (double*)malloc(1*D*sizeof(double));
  double *ytemp = (double*)malloc(1*D*sizeof(double));
  double *yfqScaled = (double*)malloc(1*D*sizeof(double));	
	
		
  for (v=0;v<V;v++){

    getMatrixRow(ytemp, Yhat, V, D, v);
					
    for (c=0;c<NC;c++){
            
      indl = v*NC+c;

      multDenseVecSparseMat(yfq,ytemp,FQ[c]);
      /*
	if(v==0 && c==0){
	displayMat(yfq,1,D);
	}
      */
				
      /*multMatrix(yfq,ytemp,1,D,mxGetPr(FQ[c]),D,D);	*/
      scaleMatrix(yfqScaled,yfq,1,D,2);
            
      setMatrixRow(mxGetPr(G),yfqScaled,NC*V,D,indl);

      Ct[indl] = dotProduct(ytemp,yfq,D) - L02[c];

    }
  }

  /*   
       printMat2File(mxGetPr(G),NC*V,D,"G.txt");
       printMat2File(Ct,NC*V,1,"Ct.txt");
  */
	
  free(yfq);
  free(ytemp);
  free(yfqScaled);

}




mxArray * updateMatrixD(mxArray *G, double *KBpK ,mwSignedIndex NC,mwSignedIndex V,mwSignedIndex D){
    
  mwIndex *irs,*jcs,i,j,iind,jind,ind;
  double *sr;
  int numCol;
  int numRow;
  int counter;
    

  mxArray *Dmat = ssmult (G, G, 0, 0, 1, 1, 0, 0); /*Dmat = G*G'*/

  numCol = mxGetN(Dmat);

  sr  = mxGetPr(Dmat);
  irs = mxGetIr(Dmat);
  jcs = mxGetJc(Dmat);

  /*mexPrintf("M %d\n",mxGetM(Dmat));*/
  /*mexPrintf("N %d\n",mxGetN(Dmat));*/

	
  counter = 0;
  for (i=0;i<numCol;i++){
    numRow = jcs[i+1]-jcs[i];
    iind = i;
    for(j=0;j<numRow;j++){
      jind = irs[counter]; 
      ind = floor(iind/NC)*V+floor(jind/NC); 
      /*mexPrintf("ind %d\n",ind); */
      sr[counter] = sr[counter]*KBpK[ind];          
      counter++;
    }
  }

  /*mexPrintf("counter %d \n",counter);*/
           
  return(Dmat);
}


mxArray * updateVectorE(double *K, double *G, double *dBp, double *Yhat,double *Ct,mwSignedIndex NC,mwSignedIndex N,mwSignedIndex V,mwSignedIndex D){
    
  mxArray *e = mxCreateDoubleMatrix(NC*V,1,mxREAL);
  mwIndex i,j,ind=0;
  double *KLine = (double*)malloc(1*(N+V)*sizeof(double));
  double *dBpKLine = (double*)malloc(D*1*sizeof(double));
  double *GLine = (double*)malloc(D*1*sizeof(double));
  double *YLine = (double*)malloc(1*D*sizeof(double));
    
    
  double *sr = mxGetPr(e);
    
  for(i=0;i<V;i++){
        
    getMatrixCol(KLine, K, N+V, N+V, i+N);
    multMatrix(dBpKLine,dBp,D,N+V,KLine,N+V,1);
        
    getMatrixRow(YLine, Yhat, V, D, i);
        
    for(j=0;j<NC;j++){
            
      getMatrixRow(GLine, G, NC*V, D, ind);
            
      sr[ind] = dotProduct(GLine,dBpKLine,D) - dotProduct(GLine,YLine,D) + Ct[ind];
            
      ind++;
    }    
        
  }
                    
   
  free(KLine);
  free(dBpKLine);
  free(GLine);
  free(YLine);
    
  return e;
}


void updateModel(double *Model,double *lms,double *G,double *K,double *d,double *Bp,int NC,int N,int V,int D){

  int i,j,ind;
  int i2;
  double *sumLmsGK = (double*) malloc(D*(N+V)*sizeof(double));
  double *KLine = (double*)malloc((N+V)*1*sizeof(double));
  double *KLineT = (double*)malloc(1*(N+V)*sizeof(double));
  double *GLine = (double*)malloc(1*D*sizeof(double));
  double *GLineT = (double*)malloc(D*1*sizeof(double));
  double *tempGK = (double*) malloc(D*(N+V)*sizeof(double));
  double *dminusGK = (double*) malloc(D*(N+V)*sizeof(double));

  /* initialize */
  setMatrixValue(sumLmsGK,D,(N+V),0);
		
  for(i=0;i<V;i++){
		
    getMatrixCol(KLine, K, N+V, N+V, i+N);
    transposeMatrix(KLineT,KLine,N+V,1);
		
    for(j=0;j<NC;j++){
			
      ind = i*NC+j;
      getMatrixRow(GLine, G, NC*V, D, ind);			 
      transposeMatrix(GLineT,GLine,1,D);
			 
      multMatrix(tempGK,GLineT,D,1,KLineT,1,N+V);			 			 

      /* sum */
      for (i2=0;i2<D*(N+V);i2++){
	sumLmsGK[i2] += tempGK[i2]*lms[ind];
      }			 
			 
    }
  }

  /*mexPrintf("max sumlmsgk = %f \n",maxMatrix(sumLmsGK,D,N+V));*/
	
	
  /* compute d - sumLmsGK */
  addScaledMatrix(dminusGK,d,sumLmsGK,D,(N+V),-1);

  /*compute the model - a = (d-sumLmsGK)*Bp */
  multMatrix(Model,dminusGK,D,N+V,Bp,N+V,N+V);
	
  free(sumLmsGK);
  free(KLine);
  free(KLineT);
  free(GLine);
  free(GLineT);
  free(tempGK);
  free(dminusGK);
	
}

/* Yhat = (a*K(:,N+1:end))'*/
void updatePredictions(double *Yhat, double *Model, double *KValid,int N,int V,int D){

  double *YhatT = (double*)malloc(V*D*sizeof(double));
	
  multMatrix(YhatT,Model,D,N+V,KValid,N+V,V);							
  transposeMatrix(Yhat,YhatT,D,V);

  free(YhatT);

}


double computeConstraintError(double *Yhat,mxArray **FQ, double *L02,int NC,int N,int V,int D){

  int i,j;
  /*double *localErrs = (double*) malloc(NC*sizeof(double));*/
  /*double *errs = (double*) malloc(V*sizeof(double));*/
  double *yfq = (double*)malloc(1*D*sizeof(double));
  double *ytemp = (double*)malloc(1*D*sizeof(double));
  double yfqy;
  double localError=0;
  double globalError=0;

	
  for (i=0;i<V;i++){

    /*setMatrixValue(localErrs,NC,1,0);*/
    getMatrixRow(ytemp, Yhat, V, D, i);
    localError = 0;	
    for(j=0;j<NC;j++){

			
      multMatrix(yfq,ytemp,1,D,mxGetPr(FQ[j]),D,D);			
      yfqy = dotProduct(yfq,ytemp,D);

      /*
	if(i==0 && j==0)
	mexPrintf("yfqy = %f\n",yfqy);
      */
			
      /*localErrs[j] = fabs(sqrt(yfqy)-sqrt(L02[j]));*/
      localError +=  fabs(sqrt(yfqy)-sqrt(L02[j]));
			
    }

    globalError += (localError/NC);

    /*errs[i] = meanMatrix(localErrs,NC,1);*/
		
  }

	
  /*free(localErrs);*/
  free(yfq);
  free(ytemp);

  return(globalError/V);

  /*return(meanMatrix(errs,V,1));*/

}

double computeConstraintError2(double *Yhat,mxArray **FQ, double *L02,int NC,int N,int V,int D){

  int i,j;
  double *fqy = (double*)malloc(1*D*sizeof(double));
  double *ytemp = (double*)malloc(1*D*sizeof(double));
  double yfqy;
  double localError=0;
  double globalError=0;

	
  for (i=0;i<V;i++){

    getMatrixRow(ytemp, Yhat, V, D, i);

    localError = 0;
    for(j=0;j<NC;j++){
			
      multSparseMatDenseVec(fqy,FQ[j],ytemp);			
      yfqy = dotProduct(fqy,ytemp,D);

      localError +=  fabs(sqrt(yfqy)-sqrt(L02[j]));

      /*
	if(i==0 && j<10){
	mexPrintf("yfqy %f\n",yfqy);
	mexPrintf("L02 %f\n",L02[j]);
	mexPrintf("err %f\n",fabs(sqrt(yfqy)-sqrt(L02[j])));
			
	}*/
		    			
    }

    /*mexPrintf("global error %f\n",localError);*/
    globalError += (localError/NC);		
  }
	
  free(fqy);
  free(ytemp);

  /*
    mexPrintf("global error %f\n",globalError);
    mexPrintf("number of const. %d\n",V);
  */

  return(globalError/V);

}



void printUsage(){

  mexPrintf("Use it for good :)  \n\n\n");

  mexPrintf("usage: \n");
  mexPrintf("prhs[0] = X labeled\n");
  mexPrintf("prhs[1] = Y labeled\n");
  mexPrintf("prhs[2] = Xhat unlabeled\n");
  mexPrintf("prhs[3] = Kernel \n");
  mexPrintf("prhs[4] = F\n");
  mexPrintf("prhs[5] = G\n");
  mexPrintf("prhs[6] = Yhat\n");
  mexPrintf("prhs[7] = Bp\n");

}


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

     
  double *X, *Y, *Xhat;	
  /* X: labeled input
     Y: labeled output
     Xhat: unlabeled input
  */
  mxArray *Yhat; /* initial estimates for the unlabeled sampels */
  double *L02; /* squared constraint values to be achived */	
  mwSignedIndex N,D,m,V,NC;	
  /* N: number of labeled data
     V: numer of unlabeled data
     D: output dimension
     m: input dimension
     NC: number of constraints per example
  */  
  int kernType;
  /* kernType = 0 : RBF
     kernType = 1 : second order polynomial
  */
  /*int numIt;*/ /* number of iterations */
  double gamma; /*regularization*/
  double invWidth; /*inverse width of the rbf kernel*/
  /*int selectBestP;*/ /* 0 - 1 to pick the best result during the iterations */

  /* temporary arrays*/
  mxArray *K; /* kernel[N+V][N+V] */
  mxArray *Xall; /* Xall = [X,Xhat] */
  mxArray *K1VAll; /* K(1:N,:) */
  mxArray *K1VAllT; /* K(:,1:N) */
  mxArray *KValid;  /*K(:,V+1:N+V)*/    
  mxArray *KValidT; /*K(V+1:N+V,:)*/
  mxArray *B;  /* K(:,1:N)*K(1:N,:)+gamma*K */
  mxArray *Bp; /*inv(B)*/
  mxArray *d;
  mxArray *dBp;
  mxArray *G;
  mxArray *Gsparse;
  mxArray *Ct;
  mxArray *KBp;
  mxArray *KBpK;

  /* temp variables */
  char buffer [100];
  int Ntemp,mtemp;
  int displayP = 0;
  mwSignedIndex mm,nm,pm;     
  mwSignedIndex i;
  mwSignedIndex NCtemp;
  mwSignedIndex NCtempN;
  mwSignedIndex Dtemp,Vtemp;

  /* variables for the linear system that computes lagrange multiplers */
  /*cs_dl *DMatSparse, Amatrix ;*/

  mxArray *lms;
  double *lmsd;
	
  clock_t start, finish;
  void *Symbolic, *Numeric;
  mxArray *Dmat;
  mxArray *e;
  mwIndex *Ai,*Ap;
  double *matData;
		
  /* constraint violation at each iteration on validation set */
  double *constErrs;
  int bestItId;

  int displayTime = 0;

  /* sanity checks */
  if(nrhs!=8){
    printUsage(); 
    sprintf(buffer,"Wrong number of inputs: %d is given\n",nrhs);
    mexErrMsgTxt(buffer);		
  }

  /*
    mexPrintf("usage: \n");
    mexPrintf("prhs[0] = X labeled\n");
    mexPrintf("prhs[1] = Y labeled\n");
    mexPrintf("prhs[2] = Xhat unlabeled\n");
    mexPrintf("prhs[3] = Kernel \n");
    mexPrintf("prhs[4] = F\n");
    mexPrintf("prhs[5] = G\n");
    mexPrintf("prhs[6] = Yhat\n");
    mexPrintf("prhs[7] = Bp\n");
  */
	
  /* read dimensions*/
  m = (mwSignedIndex)mxGetM(prhs[0]);  
  N = (mwSignedIndex)mxGetN(prhs[0]);
  D = (mwSignedIndex)mxGetM(prhs[1]);
  Ntemp = (int)mxGetN(prhs[1]);
  mtemp = (int)mxGetM(prhs[2]);
  V = (mwSignedIndex)mxGetN(prhs[2]);			
  NC = mxGetM(prhs[4])/V;
  NCtemp =  (mwSignedIndex)mxGetM(prhs[5]);
  NCtempN = (mwSignedIndex)mxGetN(prhs[5]);
  Vtemp =  (mwSignedIndex)mxGetM(prhs[6]);
  Dtemp = (mwSignedIndex)mxGetN(prhs[6]);
   
  /* read inputs X-Y-Xhat-kernType-gamma-invWidth*/
  X = mxGetPr(prhs[0]);
  Y = mxGetPr(prhs[1]);
  Xhat = mxGetPr(prhs[2]);
  K = mxDuplicateArray(prhs[3]);
  Ct = mxDuplicateArray(prhs[4]);
  Gsparse = mxDuplicateArray(prhs[5]);
  Yhat =  mxDuplicateArray(prhs[6]);
  Bp = mxDuplicateArray(prhs[7]);

  G = mxCreateDoubleMatrix(NC*V,D,mxREAL);
  convertSparse2Full(mxGetPr(G), Gsparse, NC*V, D);
			
  /* more sanity checks */
  if(N==0 || V==0){
    printUsage(); 
    sprintf(buffer,"N or V cannot be null \n");
    mexErrMsgTxt(buffer);
  }
  if(Ntemp!=N){
    printUsage(); 
    sprintf(buffer,"There are %d input [X] and %d output [Y] samples \n", (int)N,Ntemp);
    mexErrMsgTxt(buffer);
  }
  if(mtemp!=m){
    printUsage(); 
    sprintf(buffer,"Labeled input [X] dimension is %d whereas unlabeled [Xhat] dimension is %d \n",(int)m,mtemp);
    mexErrMsgTxt(buffer);
  }
  if(Dtemp!=D){
    printUsage(); 
    sprintf(buffer,"Size of predictions in Yhat(VxD)(D=%d) should be same as those in labeled set Y (%d) \n",(int)Dtemp,(int)D);
    mexErrMsgTxt(buffer);		
  }
  if(Vtemp!=V){
    printUsage(); 
    sprintf(buffer,"Number of predictions Yhat (VxD)(V=%d) should be same as the number of unlabeled samples (%d) \n",(int)Vtemp,(int)V);
    mexErrMsgTxt(buffer);
  }
   
	
  /* print input summary*/
  if(displayP==1){
    mexPrintf("\n\n",gamma);
    mexPrintf("number of labeled data= %d\n",N);
    mexPrintf("number of unlabeled data= %d\n",V);
    mexPrintf("output dimension= %d\n",D);
    mexPrintf("number of constraints per example = %d\n",NC);
    mexPrintf("\n\n");
  }

  /* combine X and Xhat */
  Xall = mxCreateDoubleMatrix(m, (N+V),  mxREAL);
  horzCatMatrices(mxGetPr(Xall),X, m, N, Xhat, m,V);
  if(displayP==1){
    mexPrintf("[X Xhat] \n");
    displayMat(mxGetPr(Xall),m,N+V);
  }

  /* /\* compute the kernel *\/ */
  /* K = mxCreateDoubleMatrix((N+V), (N+V),  mxREAL); */
  /* /\*rbf kernel*\/ */
  /* if(kernType==0){ */
  /* 	computeRBFKern(mxGetPr(K),mxGetPr(Xall),mxGetPr(Xall),m,(N+V),(N+V),invWidth); */
  /* 	if(displayP==1){ */
  /* 		mexPrintf("K \n"); */
  /* 		displayMat(mxGetPr(K),(N+V),(N+V)); */
  /* 	} */
  /* } */
  /* /\* linear kernel *\/ */
  /* if(kernType==1){ */
  /* 	computeLinKern(mxGetPr(K),mxGetPr(Xall),mxGetPr(Xall),m,(N+V),(N+V)); */
  /* 	if(displayP==1){ */
  /* 		mexPrintf("K \n"); */
  /* 		displayMat(mxGetPr(K),(N+V),(N+V)); */
  /* 	} */
  /* } */

  /* extract K(1:N,:) */
  K1VAll =  mxCreateDoubleMatrix(N,(N+V),mxREAL);
  getMatrixBlock(mxGetPr(K1VAll), mxGetPr(K), (N+V), (N+V), 0, N-1, 0, (N+V)-1);
  if(displayP==1){
    mexPrintf("K(1:N,:) \n");
    displayMat(mxGetPr(K1VAll),N,(N+V));
  }
	
  /* extract K(:,1:V) */
  K1VAllT =  mxCreateDoubleMatrix((N+V),N,mxREAL);
  transposeMatrix(mxGetPr(K1VAllT),mxGetPr(K1VAll),N,(N+V));
    
    
  /* extract K(:,N+1:end) and K(N+1:end,:)*/
  KValid = mxCreateDoubleMatrix((N+V),V,mxREAL);
  KValidT = mxCreateDoubleMatrix(V,(N+V),mxREAL);
  getMatrixBlock(mxGetPr(KValid), mxGetPr(K), (N+V), (N+V), 0, N+V-1, N , N+V-1);
  transposeMatrix(mxGetPr(KValidT),mxGetPr(KValid),(N+V),V);
        
  if(displayP==1){
    mexPrintf("K(:,1:N) \n");
    displayMat(mxGetPr(K1VAllT),(N+V),N);
    printMat2File(mxGetPr(K1VAllT),(N+V),N,"temp.txt");
  }

  /* compute B */
  /* B =  mxCreateDoubleMatrix((N+V),(N+V),mxREAL); */
  /* multMatrix(mxGetPr(B),mxGetPr(K1VAllT),(N+V),N,mxGetPr(K1VAll),N,(N+V)); */
  /* addScaledMatrix(mxGetPr(B),mxGetPr(B),mxGetPr(K),(N+V),(N+V),gamma); */
  /* if(displayP==1){ */
  /* 	mexPrintf("B \n"); */
  /* 	displayMat(mxGetPr(B),(N+V),(N+V)); */
  /* } */

  /* compute Bp = inv(B) */
  /* Bp = mxCreateDoubleMatrix((N+V),(N+V),mxREAL); */
  /* invertMatrix(mxGetPr(Bp), mxGetPr(B), (N+V)); */
  /* if(displayP==1){ */
  /* 	mexPrintf("Bp \n"); */
  /* 	displayMat(mxGetPr(Bp), (N+V), (N+V)); */
  /* } */
    
  /* compute KBpK */
  KBp = mxCreateDoubleMatrix(V,(N+V),mxREAL);
  multMatrix(mxGetPr(KBp),mxGetPr(KValidT),V,(N+V),mxGetPr(Bp),(N+V),(N+V));
  KBpK = mxCreateDoubleMatrix(V,V,mxREAL);
  multMatrix(mxGetPr(KBpK),mxGetPr(KBp),V,(N+V),mxGetPr(KValid),(N+V),V);
  if(displayP==1){
    mexPrintf("KBpK \n");
    displayMat(mxGetPr(KBpK), V, V);
  }

  /* compute d */
  d = mxCreateDoubleMatrix(D,(N+V),mxREAL);
  multMatrix(mxGetPr(d),Y,D,N,mxGetPr(K1VAll),N,N+V);
    
  /* compute dBp */
  dBp = mxCreateDoubleMatrix(D,(N+V),mxREAL);
  multMatrix(mxGetPr(dBp),mxGetPr(d),D,N+V,mxGetPr(Bp),N+V,N+V);

  /* initialize lagrange multipliers */
  /*lms = mxCreateDoubleMatrix(NC*V,1,mxREAL);
    lmsd = mxGetPr(lms);*/

  /* /\* initialize the predictor model *\/ */
  /* mxArray * Model = mxCreateDoubleMatrix(D,(N+V),mxREAL);  */

	
  /* update Dmat and e */
	
  start = clock();	
  Dmat = updateMatrixD(Gsparse, mxGetPr(KBpK),NC,V,D);			
  e = updateVectorE(mxGetPr(K),mxGetPr(G),mxGetPr(dBp),mxGetPr(Yhat),mxGetPr(Ct),NC,N,V,D);
  finish = clock();
  if(displayTime>0)
    mexPrintf("updating D and e takes %.2lf seconds \n", difftime(finish,start)/CLOCKS_PER_SEC );

	           	
  /* /\* outputs*\/ */
  plhs[0] = mxDuplicateArray(Dmat); 
  plhs[1] = mxDuplicateArray(e); 
	
  
  /* destroy temporary arrays */

	
  mxDestroyArray(Xall);
  mxDestroyArray(K1VAll);
  mxDestroyArray(K1VAllT);
  mxDestroyArray(KValid);
  mxDestroyArray(KValidT);
  mxDestroyArray(d);	
  mxDestroyArray(dBp);
  mxDestroyArray(G);

  mxDestroyArray(KBp);
  mxDestroyArray(KBpK);
	
	    
  return;					    
}
