function errs = computeReconstructionError(yp,ygt,v2vP)

% Copyright (C) <2012>  <Aydin Varol, Mathieu Salzmann, Pascal Fua and Raquel Urtasun>
% 
% Implementation of CLVM method described in:
% A. Varol, M. Salzmann, P. Fua and R. Urtasun, A Constrained Latent Variable Model, 
% Conference on Computer Vision and Pattern Recognition, June 2012.
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Aydin Varol, CVLab-EPFL, 2012.
% aydin.varol@gmail.com, www.aydinvarol.com

computeVert2Vert = 0;
if(exist('v2vP','var'))
    if(v2vP>0)
        computeVert2Vert = 1;
    end
end
  
if(iscell(yp))
    yp = cell2mat(yp);
end
if(iscell(ygt))
    ygt = cell2mat(ygt);
end

errs = zeros(size(yp,1),1);

if(size(yp,1)~=size(ygt,1))
    error('size mismatch in error function.');
end

N = size(ygt,2);

if(computeVert2Vert>0 && mod(N,3)>0)
    error('cannot use vert2vert distance while computing error.');
end

for i = 1 : size(yp,1)
    ygtt = ygt(i,:);
    if(computeVert2Vert==0)
        errs(i) = sqrt(sum((yp(i,:)-ygtt).^2));
    else
        localErrs = zeros(N/3,1);        
        for k = 1 : N/3
            localErrs(k) = norm(yp(i,(k-1)*3+1:3*k)-ygtt((k-1)*3+1:3*k));
        end
        errs(i) = mean(localErrs);
    end
end


