function modelCbest = learnConstraintKernelIneqMex(X,Y,Xhat,myKern,constrFunHandle,Yhat,Y0,gamma,alpha,numIt,selectBestP,displayP)

% Copyright (C) <2012>  <Aydin Varol, Mathieu Salzmann, Pascal Fua and Raquel Urtasun>
% 
% Implementation of CLVM method described in:
% A. Varol, M. Salzmann, P. Fua and R. Urtasun, A Constrained Latent Variable Model, 
% Conference on Computer Vision and Pattern Recognition, June 2012.
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Aydin Varol, CVLab-EPFL, 2012.
% aydin.varol@gmail.com, www.aydinvarol.com

t1 = tic;

sparsityThreshold = 0.3;

if(isempty(X) || isempty(Xhat))
    error('number of labeled or unlabeled data cannot be zero');
end

NC = size(constrFunHandle(Yhat(1,:)),1);
N = size(X,1);
V = size(Xhat,1);
D = size(Y,2);
X = X';
Y = Y';
Xhat = Xhat';

initialEqConstErrors = zeros(NC*V,1);
for i = 1:size(Yhat,1)
    initialEqConstErrors((i-1)*NC+1:i*NC) = constrFunHandle(Yhat(i,:));
end


if(displayP)
    fprintf('It. %d, ineq. constr. error on validation set %f \n',0,mean(computeIneqConstError(Yhat,constrFunHandle)));
end

if(iscell(Yhat))
    Yhat = cell2mat(Yhat);
end

K = myKern([X Xhat],[X Xhat]);
B = K(:,1:N)*K(1:N,:)+gamma*K;
Bp = pinv(B);
d = Y*K(1:N,:);

G = zeros(NC*V,D);
C0 = zeros(NC*V,1);
modelC = cell(numIt,1);
ce = zeros(numIt,1);
sumgammaGK = zeros(D,N+V);

Yhattemp = Yhat;

et = initialEqConstErrors;

for ii = 1:numIt
    
   
    for i=1:V
        
        ind1 = (i-1)*NC+1;
        ind2 = i*NC;
        
        [temp1 temp2] = constrFunHandle(Yhattemp(i,:));
        
        C0(ind1:ind2,:) = temp1;
        G(ind1:ind2,:) = temp2;
        
    end

    G = sparse(G);  
    
    
    [D2 e2] = buildLinSys(X,Y,Xhat,K,C0,G,Yhattemp-repmat(Y0,V,1),Bp);
    
    D2 = D2 + sparse(diag((et.*et/alpha)));
    e2 = e2 - 0.5*(et.*et);
    
    if(howSparse(D2)<sparsityThreshold && displayP)
        fprintf('constraint matrix is not sparse (sparsity=%0.2f). \n',howSparse(D2));
        D2 = full(D2);
    end
    
    if(issparse(D2))
        lm = D2\e2;
    else
        lm = pinv(D2)*e2;
    end        
    
    et = -(lm.*et)/alpha; 
    
    sumgammaGK = 0 * sumgammaGK;
    for i=1:V
        for j=1:NC
            indl = (i-1)*NC+j;
            sumgammaGK = sumgammaGK + lm(indl)*G(indl,:)'*K(:,i+N)';
        end
    end  
    
    modelC{ii}.a = (d-sumgammaGK)*Bp;
           
    Yhattemp = (modelC{ii}.a*K(:,N+1:end))'+repmat(Y0,V,1);
   
    ce(ii) = mean(computeIneqConstError(Yhattemp,constrFunHandle));
    
    %YpredLabeled = (modelC{ii}.a*K(:,1:N))'+repmat(Y0,N,1);
    %temp = computeReconstructionError(YpredLabeled,Y'+repmat(Y0,N,1),1);
    
    if(displayP)
        fprintf(2,'It. %d, ineq. constr. error on validation set %f \n',ii,ce(ii));
        %fprintf(1,'It. %d, recons. error on labeled set %f \n',ii,mean(temp));
    end
   
end

bestit = numIt;
if(selectBestP)
    [best bestit] = min(ce);
    modelCbest = modelC{bestit};
    if(displayP)
        fprintf(2,'\n Best iteration: %d \n\n',bestit);
    end
else
    modelCbest = modelC{numIt};
end


modelCbest.runTime = toc(t1);
modelCbest.Xlabeled = X';
modelCbest.Ylabeled = Y';
modelCbest.Xunlabeled = Xhat';
modelCbest.Yhat = Yhat;
modelCbest.K = K;
modelCbest.Kinv = inv(K);
modelCbest.myKern = myKern;
modelCbest.gamma = gamma;
modelCbest.Y0 = Y0;
modelCbest.bestIt = bestit;
modelCbest.numIt = numIt;

clear modelC;
clear G;
clear K;
clear Bp;
clear d;
clear sumgammaGK;



