function WC = learnConstraintW(X,Y,Xhat,My,Sy,lambda,F,c,WLeastSquare,maxNumberIt,selectBest,displayP)

% Copyright (C) <2012>  <Aydin Varol, Mathieu Salzmann, Pascal Fua and Raquel Urtasun>
% 
% Implementation of CLVM method described in:
% A. Varol, M. Salzmann, P. Fua and R. Urtasun, A Constrained Latent Variable Model, 
% Conference on Computer Vision and Pattern Recognition, June 2012.
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the version 3 of the GNU General Public License
% as published by the Free Software Foundation.
% 
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
% General Public License for more details.       
% You should have received a copy of the GNU General Public License
% along with this program. If not, see <http://www.gnu.org/licenses/>.
%
% Aydin Varol, CVLab-EPFL, 2012.
% aydin.varol@gmail.com, www.aydinvarol.com

stepSize = 1;
it = 1;
exitFlag = 0;

N = size(Y,2);
NM = size(kron(speye(N),X(1,:)),2);

nv = size(Xhat,1);
nd = size(X,1);
A = zeros(nd*N+NM,NM);
b = zeros(nd*N+NM,1);

for i = 1 : nd    
    A((i-1)*N+1:i*N,:) = sparse(kron(speye(N),X(i,:)));
    b((i-1)*N+1:i*N,:) = Y(i,:)';            
end
A(nd*N+1:end,:) = sqrt(lambda)*speye(NM); % for all w's
A = sparse(A);

Xv = zeros(nv*N,NM);
for i = 1 : nv
    Xv((i-1)*N+1:i*N,:) = sparse(kron(speye(N),Xhat(i,:)));
end
Xv = sparse(Xv);

wt = WLeastSquare;


constErrs = zeros(maxNumberIt,1);
wts = cell(maxNumberIt,1);

while it<=maxNumberIt && exitFlag==0
    
           
    B = zeros(size(Xhat,1)*length(c),NM);
    C = zeros(size(Xhat,1)*length(c),1);
     
    % for each validation sample
    for vs = 1 : size(Xhat,1)
        
        xx = Sy*Xv((vs-1)*N+1:vs*N,:);     
        
        % for each constraint of the sample
        for es = 1 : length(c)
            
            L02 = (xx*wt)'*F{es}*(xx*wt);
            LShift2 = My'*F{es}*My;
            L2 = c(es)^2;
            
            B((vs-1)*length(c)+es,:) = sparse(2*(xx*wt)'*F{es}*xx + 2*My'*F{es}*xx);
            C((vs-1)*length(c)+es,:) = L2-L02-LShift2-2*(xx*wt)'*F{es}*My;             
            
        end
    end        
    
    % compute the null matrix, V and a particular solution, vStar
    V = null(B);
    vStar = B\C;
  
    % solve for the data term using the new variables, gamma
    gamma = A*V\(b-A*(wt+vStar));

     
    % update the estimate, wt
    wt = wt + vStar*stepSize + V*gamma*stepSize;
    
    ypr = predictW(wt,Xhat,My,Sy);
    
    
    ceee = zeros(size(ypr,1),1);    
    for j1 = 1 : size(ypr,1)
        localErr = zeros(length(c),1);
        for j2 = 1 : length(c)
            localErr(j2) = ypr(j1,:)*F{j2}*ypr(j1,:)'-c(j2)^2;        
        end
        ceee(i) = mean(abs(localErr));        
    end
                
    
    newConstError = mean(ceee);

  
    constErrs(it) = newConstError;
    wts{it} = wt;
    
    if(displayP)
        fprintf('it=%d, validConstErr = %d, \n',it,newConstError);
    end
    
    it = it + 1;
    
end

%figure;plot(constErrs);
if(selectBest==0)
    WC = wt;
else
    [t1 t2] = min(constErrs);
    WC = wts{t2};
end

